# Generated by roxygen2: do not edit by hand

export(BatchContainer)
export(BatchContainerDimension)
export(L1_norm)
export(L2s_norm)
export(accept_leftmost_improvement)
export(assign_from_table)
export(assign_in_order)
export(assign_random)
export(batch_container_from_table)
export(compile_possible_subgroup_allocation)
export(complete_random_shuffling)
export(drop_order)
export(first_score_only)
export(form_homogeneous_subgroups)
export(generate_terms)
export(get_order)
export(mk_exponentially_weighted_acceptance_func)
export(mk_plate_scoring_functions)
export(mk_simanneal_acceptance_func)
export(mk_simanneal_temp_func)
export(mk_subgroup_shuffling_function)
export(mk_swapping_function)
export(optimize_design)
export(optimize_multi_plate_design)
export(osat_score)
export(osat_score_generator)
export(plot_plate)
export(shuffle_grouped_data)
export(shuffle_with_constraints)
export(shuffle_with_subgroup_formation)
export(sum_scores)
export(worst_score)
import(R6, except = getNamespaceExports("R6"))
importFrom(rlang,.data)
importFrom(stats,na.omit)
